# coding: utf-8

"""
    Bare Metal Cloud API

    Create, power off, power on, reset, reboot, or shut down your server with the Bare Metal Cloud API.  Deprovision servers, get or edit SSH key details, assign public IPs, assign servers to networks and a lot more.  Manage your infrastructure more efficiently using just a few simple API calls.<br> <br> <span class='pnap-api-knowledge-base-link'> Knowledge base articles to help you can be found <a href='https://phoenixnap.com/kb/how-to-deploy-bare-metal-cloud-server' target='_blank'>here</a> </span><br> <br> <b>All URLs are relative to (https://api.phoenixnap.com/bmc/v1/)</b> 

    The version of the OpenAPI document: 0.1
    Contact: support@phoenixnap.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, ClassVar, Dict, List
from pydantic import BaseModel, StrictStr, field_validator
from pydantic import Field
from typing_extensions import Annotated
from pnap_bmc_api.models.quota_edit_limit_request_details import QuotaEditLimitRequestDetails
try:
    from typing import Self
except ImportError:
    from typing_extensions import Self

class Quota(BaseModel):
    """
    Quota.
    """ # noqa: E501
    id: StrictStr = Field(description="The ID of the Quota.")
    name: StrictStr = Field(description="The name of the Quota.")
    description: StrictStr = Field(description="The Quota description.")
    status: StrictStr = Field(description="The status of the quota resource usage.")
    limit: Annotated[int, Field(strict=True, ge=0)] = Field(description="The limit set for the quota.")
    unit: StrictStr = Field(description="Unit of the quota type. Supported values are 'COUNT' and 'GB'.")
    used: Annotated[int, Field(strict=True, ge=0)] = Field(description="The quota used expressed as a number.")
    quota_edit_limit_request_details: List[QuotaEditLimitRequestDetails] = Field(alias="quotaEditLimitRequestDetails")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["id", "name", "description", "status", "limit", "unit", "used", "quotaEditLimitRequestDetails"]

    @field_validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('WITHIN_LIMIT', 'OVER_LIMIT', 'ON_LIMIT'):
            raise ValueError("must be one of enum values ('WITHIN_LIMIT', 'OVER_LIMIT', 'ON_LIMIT')")
        return value

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Quota from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={
                "quota_edit_limit_request_details",
                "additional_properties",
            },
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in quota_edit_limit_request_details (list)
        _items = []
        if self.quota_edit_limit_request_details:
            for _item in self.quota_edit_limit_request_details:
                if _item:
                    _items.append(_item.to_dict())
            _dict['quotaEditLimitRequestDetails'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of Quota from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "status": obj.get("status"),
            "limit": obj.get("limit"),
            "unit": obj.get("unit"),
            "used": obj.get("used"),
            "quotaEditLimitRequestDetails": [QuotaEditLimitRequestDetails.from_dict(_item) for _item in obj.get("quotaEditLimitRequestDetails")] if obj.get("quotaEditLimitRequestDetails") is not None else None
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


